-------------------------- MODULE InitialStates ---------------------------
\* This module defines the initial state and type invariants

(* The hierarchy of the modules are
 Ring (The basic parameters and operations for ring calculation)
   \subset LS (The leaf set utilities)
      \subset RT (The routing table utilities)
         \subset Msg (The messages definitions)
            \subset InitialStates (The initial state and TypeInvariant)
               \subset Actions (All the actions)
                  \subset MSPastry (The specification of MSPastry including properties)
                     \subset MCMSPastry (The demonstration assignments for MSPastry)
*)
EXTENDS Msg
VARIABLE 
   receivedMsgs, (* To maintain the pool of unhandled message*)
   active,       \* To record the statuses of a node
   rtable,       \* To store the routing table of the nodes
   lset,         \* To store the leaf set of the nodes
   probing,      \* To keep the trace of each node, which nodes they are probing
   failed        \* To mark the unreachable/non-reactive nodes after the node has failed probing them
------------------------------------------------------------------------------
Init == 
  /\ receivedMsgs = {}
  /\ active = [i \in I |-> IF i \in A THEN TRUE ELSE FALSE] 
  /\ lset = [i \in I |->  IF i \in A
                          THEN AddToLSet(A, EmptyLS(i))
                          ELSE EmptyLS(i)]
  /\ rtable = [i \in I |-> IF i \in A 
                           THEN AddToTable(A, InitRTable, i)
                           ELSE AddToTable({i}, InitRTable, i)]

  /\ probing = [i \in I |-> {}]
  /\ failed = [i \in I |-> {}]
---------------------------------------------------------------------------
TypeInvariant == 
  /\ receivedMsgs \in SUBSET DMsg
  /\ active \in [I -> BOOLEAN]
  \*/\ cover \in [I -> [left: Nat, right: Nat]]
  /\ rtable \in [I -> RTable]
  /\ lset \in [I -> LSet]
  /\ probing  \in [I -> SUBSET I]
  /\ failed \in [I -> SUBSET I]


==============================================================
