-------------------------- MODULE MCMSPastry ---------------------------
\* This is the model checking module for specification MSPastry
(* The hierarchy of the modules are
 Ring (The basic parameters and operations for ring calculation)
   \subset LS (The leaf set utilities)
      \subset RT (The routing table utilities)
         \subset Msg (The messages definitions)
            \subset InitialStates (The initial state and TypeInvariant)
               \subset Actions (All the actions)
                  \subset MSPastry (The specification of MSPastry including properties)
                     \subset MCMSPastry (The demonstration assignments for MSPastry)
*)
EXTENDS MSPastry
------------------------------------------------------------------------
\* Demo Assignment
MCI == {17, 55, 65, 95} \* We model checking 4 nodes system 
MCA == {17} \* Initial TRUE node is 17
MCB == 4 \* base is 2^4
MCM == 8 \* ID space is 2^8
MCL == 1 \* leaf set length is 2, left one node, right one node
CastroNext == \E i, j \in I:
      \/ RouteJReq(i, j)   
      \/ RecProbe(i)
      \/ RecPRply(i) 
      \/ CastroRecJReq(i)
      \/ RecJReply(i)
      \/ CastroDeliver(i, j)
      \/ RouteLookup(i, j) 
      \*\/ MsgLost
      \* followed are application (interface) actions:
      \*\/ Join(i, j)
      \*\/ Lookup(i, j)

----------------------------------------------------------------
\* The following are liveness properties and their specification configurations
(*Successful lookup*)
MCALookup == {17, 65}
---------------
\* Castro lookup without faulty
CastroLookupNext == \/ Lookup(17, 95)
                    \/ CastroNext
SpecSuccessLookup ==
/\ Init 
/\ [][CastroLookupNext]_vars

\*property to refute
CastroNeverDeliver == [][\A i, j \in I: ~CastroDeliver(i, j)]_vars
--------------------------------------------------------------
(*Successful join*)
\* Castro pure join
JoinNext == \/ CastroJoin(95, 17)
            \/ CastroNext
SpecSuccessJoin == 
/\ Init 
/\ [][JoinNext]_vars

\*property to refute
CastroNeverJoin == [](\A j \in I \ A: active[j]#TRUE) 
-------------------------------------------------------
(*Successful Concjoin*)
MCCJA == {17, 95}
---------------------------
\*Castro pure join
CJNext ==  \/ CastroJoin(65, 95)
           \/ CastroJoin(55, 17)
           \*\/ Join(70, 95)           
           \/ CastroNext
SpecConcJoin == 
/\ Init 
/\ [][CJNext]_vars           

----------------------
\*property to refute  
CastroConcurrentJoin == []~(active[95] = TRUE 
                          /\active[17] = TRUE
                          /\active[65]= TRUE 
                          /\active[55]= TRUE
                          )
---------------------------------------------------------
(*Successful Concjoin and then lookup*)
\*Castro pure join
CcJoinDeliverNext == \/ CastroJoin(95, 17)
                     \/ CastroJoin(55, 17)
                     \/ Lookup(95, 65)
                     \/ CastroNext
SpecCcJoinDeliver == 
/\ Init 
/\ [][CcJoinDeliverNext]_vars

----------------------
\*property to refute  
CastroCcJoinDeliver ==[][~\E i, j \in I: ENABLED CastroDeliver(i, j) 
                          /\active[95]= TRUE 
                          /\active[17]= TRUE 
                          /\active[55]= TRUE]_vars
--------------------------------
(*Safety properties*)
MCANCF == {17, 95}
---------------------------
\*Castro pure concurrent join

NCFNext == \/ CastroJoin(65, 95)
           \/ CastroJoin(55, 17)
           \/ Lookup(55, 65)
           \/ Lookup(65, 65)
           \/ CastroNext
CastroSpecNCF ==
/\ Init 
/\ [][NCFNext]_vars
LNext == \E i, j \in I:
      \/ RouteJReq(i, j)   
      \/ RecProbe(i)
      \/ RecPRply(i) 
      \/ CastroRecJReq(i)
      \/ RecJReply(i)
      \/ CastroDeliver(i, j)
      \/ RouteLookup(i, j) 
      \/ Lookup(i, j)
NCFRealNext == \/ CastroJoin(65, 95)
               \/ CastroJoin(55, 17)
               \/ LNext      
CastroSpecNCFReal ==
/\ Init 
/\ [][NCFRealNext]_vars
-----------------------------------------------------------------------
\* Properties 
 
CastroCorrectDelivery == \A i,k \in I: 
  ENABLED CastroDeliver(i, k)
  => /\ \A n \in I\{k}: active[n] = TRUE => AbsDist(i, k) =< AbsDist(n, k)
     /\ \A j \in I\{i}: ~ENABLED CastroDeliver(j, k) 

=======================================================================
